<?php

require_once __DIR__."/../vendor/autoload.php";

use NoccyLabs\InteractiveShell\Shell;
use NoccyLabs\InteractiveShell\Context\ShellContext;
use NoccyLabs\InteractiveShell\Command\Commands\QuitCommand;
use NoccyLabs\InteractiveShell\Command\Commands\HelpCommand;
use NoccyLabs\InteractiveShell\Command\Commands\Command;
use NoccyLabs\InteractiveShell\Command\ShellCommand;

class TestCommand extends ShellCommand {

    protected function build(): void {
        $this->setCommandName("test");
        $this->setDescription("This is a test command");
    }

    public function invoke(array $args, ShellContext $context): void {
        // note how we can use echo and friends here, since output is buffered
        echo "Hello World!\n";
        echo "<options=bold>This text is bold</options=bold> but <bold>so is this!</>\n";
        echo "There is also <dim>dim</>, <italic>italic</>, <underline>underline</>!\n";
        echo "<fg=red>Red text</> <fg=black;bg=yellow>Black on Yellow</>\n";
    }

}

class RootContext extends ShellContext {

    private $process = null;

    protected function build(): void {
        $this->addCommand(new HelpCommand());
        $this->addCommand(new QuitCommand());
        $this->addCommand(new Command("exec", "Execute a command", 
            function (array $args, ShellContext $context) {
                if ($context->process) return;
                $cmd = escapeshellcmd(array_shift($args));
                $arg = array_map("escapeshellarg", $args);
                $cmd = trim($cmd." ".join(" ",$arg));
                $context->process = popen($cmd, "r");
            })
        );
    }

    public function contextAttached(): void {
        $this->getShell()->setPrompt("exectest> ");
    }

    public function update(): void
    {
        if (!$this->process) return;
        $r = [$this->process]; $w = $e = null;
        if (stream_select($r,$w,$e,0)) {
            $read = fread($this->process, 8192);
            if ($read) {
                echo $read;
            } else {
                pclose($this->process);
                $this->process = null;
            }
        }
    }

}

class MyShellRunner {

    private Shell $shell;

    private ShellContext $context;

    public function startShell(): void {
        $this->context = new RootContext();
        $this->shell = new Shell();
        $this->shell->run($this->context);
    }

}

$shell = new MyShellRunner();
$shell->startShell();

