<?php

require_once __DIR__."/../vendor/autoload.php";

use NoccyLabs\InteractiveShell\Shell;
use NoccyLabs\InteractiveShell\Context\ShellContext;
use NoccyLabs\InteractiveShell\Command\Commands\QuitCommand;
use NoccyLabs\InteractiveShell\Command\Commands\HelpCommand;
use NoccyLabs\InteractiveShell\Command\Commands\Command;
use NoccyLabs\InteractiveShell\Command\ShellCommand;

class TestCommand extends ShellCommand {

    protected function build(): void {
        $this->setCommandName("test");
        $this->setDescription("This is a test command");
    }

    public function invoke(array $args, ShellContext $context): void {
        // note how we can use echo and friends here, since output is buffered
        echo "Hello World!\n";
        echo "<options=bold>This text is bold</options=bold> but <bold>so is this!</>\n";
        echo "There is also <dim>dim</>, <italic>italic</>, <underline>underline</>!\n";
        echo "<fg=red>Red text</> <fg=black;bg=yellow>Black on Yellow</>\n";
    }

}

class RootExtension extends ShellContext {

    protected function build(): void {
        $this->addCommand(new TestCommand());
    }

}

class RootContext extends ShellContext {

    private $process = null;

    protected function build(): void {
        $this->addCommand(new HelpCommand());
        $this->addCommand(new QuitCommand());
    }

    public function contextAttached(): void {
        $this->getShell()->setPrompt("extensiontest> ");
    }

    public function update(): void
    {
    }

}

class MyShellRunner {

    private Shell $shell;

    private ShellContext $context;

    public function startShell(): void {
        $this->context = new RootContext();
        $this->context->addContextExtension(new RootExtension());
        $this->shell = new Shell();
        $this->shell->run($this->context);
    }

}

$shell = new MyShellRunner();
$shell->startShell();

