<?php

namespace NoccyLabs\InteractiveShell\Command\Commands;

use NoccyLabs\InteractiveShell\Command\ShellCommand;
use NoccyLabs\InteractiveShell\Context\ShellContext;

class HelpCommand extends ShellCommand
{
    protected function build(): void
    {
        $this->setCommandName("help");
        $this->setDescription("Show help");
    }

    public function invoke(array $args, ShellContext $context)
    {
        $command = array_shift($args);
        if (!$command) {
            $this->printCommandList($context);
        } else {
            $this->printCommandInfo($context, $command);
        }
    }

    private function printCommandList(ShellContext $context)
    {
        $commands = $context->findAllCommands();
        echo "<options=bold>Available commands:</>\n";
        foreach ($commands as $command) {
            echo "  <fg=yellow>".str_pad($command->getCommandName(),20)."</> ".$command->getDescription()."\n";
        }
    }

    private function printCommandInfo(ShellContext $context,string $command)
    {
        $commandInfo = $context->findCommand($command);
        if (!$commandInfo) {
            echo "No such command: {$command}\n";
            return;
        }
        echo "<bold>".$commandInfo->getCommandName()."</> - <italic>".$commandInfo->getDescription()."</>\n";
    }
}