<?php

namespace NoccyLabs\InteractiveShell\Command\Commands;

use NoccyLabs\InteractiveShell\Command\ShellCommand;
use NoccyLabs\InteractiveShell\Context\ShellContext;
use NoccyLabs\InteractiveShell\Option\AbstractOptionSet;
use NoccyLabs\InteractiveShell\Option\OptionSet;

class SetCommand extends ShellCommand
{
    private AbstractOptionSet $options;

    public function __construct(AbstractOptionSet $options)
    {
        $this->options = $options;

        parent::__construct();
    }

    protected function build(): void
    {
        $this->setCommandName("set");
        $this->setDescription("Set a configuration value");
    }

    public function invoke(array $args, ShellContext $context)
    {
        $key = array_shift($args);
        $value = array_shift($args);
        if (!$value && str_contains($key,"=")) {
            [$key,$value] = explode("=", $key, 2);
        }

        if ($value !== null) {
            $this->setValue($key, $value);
            $this->printValue($key);
        } elseif ($key !== null) {
            $this->printValue($key);
        } else {
            $this->printAllValues();
        }
    }

    private function setValue(string $key, mixed $value)
    {
        $this->options->set($key, $value);
    }

    private function printValue(string $key)
    {
        try {
            $val = $this->options->get($key);
            $this->print($key,$val);
        } catch (\Exception $e) {
            echo "Invalid key\n";
        }
    }

    private function printAllValues()
    {
        $opts = $this->options->getOptions();
        foreach ($opts as $k=>$v) {
            $this->print($k,$v->getValue(), $v->getComment());
            //echo "  <fg=cyan>{$k}</> = <fg=yellow>{$v}\n";
        }
    }

    private function print(string $key, mixed $value, ?string $comment=null)
    {
        echo "  <fg=cyan>{$key}</>=<fg=yellow>".json_encode($value,JSON_UNESCAPED_SLASHES);
        if ($comment) echo " <fg=green># {$comment}</>";
        echo "\n";

    }
}
