<?php

namespace NoccyLabs\InteractiveShell\Input;

use NoccyLabs\InteractiveShell\Shell;

class ConsoleInput implements InputInterface
{

    private $oldMode;

    private Shell $shell;

    private InputBuffer $buffer;

    public function __construct(Shell $shell, InputBuffer $buffer)
    {
        $this->shell = $shell;
        $this->buffer = $buffer;

        $this->oldMode = exec("stty -g");
        exec("stty cbreak -echo");
    }

    public function __destruct()
    {
        exec("stty {$this->oldMode}");
    }

    public function readInput(): ?string
    {
        $r = [ STDIN ]; $w = $e = null;
        if (stream_select($r,$w,$e,0)) {
            $b = fread(STDIN, 8182);
            return $b;
        }
        return null;
    }

    public function update(): void
    {
        $buf = $this->readInput();
        if ($buf === null) return;
        
        //echo "read: [".strlen($buf)."]\n";
        while (mb_strlen($buf)>0) {
            if (str_starts_with($buf,"\e")) {
                if (preg_match("<^\e\[([0-9~;]*)([a-z]{1})>i", $buf, $match)) {
                    $key = $match[0];
                    $buf = mb_substr($buf, mb_strlen($key));
                    switch ($key) {
                        case "\e[A": $this->buffer->historyBack(); break;
                        case "\e[B": $this->buffer->historyNext(); break;
                        case "\e[C": $this->buffer->setCursorRelative(1); break;
                        case "\e[D": $this->buffer->setCursorRelative(-1); break;
                        case "\e[H": $this->buffer->setCursorHome(); break;
                        case "\e[F": $this->buffer->setCursorEnd(); break;
                        default: echo json_encode($key)."\n";break;
                    }
                    continue;
                }
                echo "error parsing: [".json_encode($buf)."]\n";
                break;
            } else {
                $ch = mb_substr($buf, 0, 1);
                $buf = mb_substr($buf, 1);
                if (mb_ord($ch) === 127) {
                    $this->buffer->erasePrevious();
                } elseif (mb_ord($ch) === 10) {
                    $buffer = $this->buffer->getBuffer();
                    $this->shell->executeBuffer($buffer);
                    $this->buffer->clearBuffer();
                    break;
                } elseif (ord($ch)>=32 && ord($ch)<127) {
                    $this->buffer->insertAndAdvance($ch);
                } else {
                    echo "[".ord($ch)."]\n";
                }
            }
        }
    }
}