<?php

namespace NoccyLabs\InteractiveShell\Input;

class InputBuffer
{


    private string $prompt = "<fg=cyan>shell></> ";

    private string $buffer = "";

    private array $history = [];

    private ?int $historyIndex = null;

    private ?string $historyStash = null;

    private int $cursorPosition = 0;

    private bool $updated = true;

    public function setPrompt(string $prompt)
    {
        $this->prompt = $prompt;
        $this->updated = true;
    }

    public function getPrompt(): string
    {
        return $this->prompt;
    }

    public function setBuffer(string $buffer)
    {
        $this->buffer = $buffer;

        // Make sure the cursor are within bounds
        $this->setCursorRelative(0);

        $this->updated = true;
    }

    public function clearBuffer()
    {
        $this->setBuffer("");
        $this->updated = true;
    }

    public function getBuffer(): string
    {
        return $this->buffer;
    }

    public function getBufferLength(): int
    {
        return mb_strlen($this->buffer);
    }

    public function historyBack(): void
    {
        // If there is no more history, do nothing
        if ($this->historyIndex >= count($this->history) - 1) {
            return;
        }

        // If there is no histry, do nothing
        if (count($this->history) == 0) {
            return;
        }

        // Stash current buffer
        if ($this->historyIndex === null) {
            $this->historyStash = $this->buffer;
            $this->historyIndex = 0;
        } else {
            $this->historyIndex++;
        }
        $this->setBuffer($this->history[$this->historyIndex]);
    }

    public function historyNext(): void
    {
        // If there is no histry, do nothing
        if (count($this->history) == 0) {
            return;
        }

        // Stash current buffer
        if ($this->historyIndex <= 0) {
            $this->setBuffer((string)$this->historyStash);
            $this->historyStash = null;
            $this->historyIndex = null;
        } else {
            $this->historyIndex--;
            $this->setBuffer($this->history[$this->historyIndex]);
        }
    }

    public function setCursorHome(): void
    {
        $this->cursorPosition = 0;
        $this->updated = true;
    }

    public function setCursorEnd(): void
    {
        $this->cursorPosition = mb_strlen($this->buffer);
        $this->updated = true;
    }

    public function erasePrevious()
    {
        if ($this->cursorPosition <= 0) return;
        $this->buffer
            = mb_substr($this->buffer, 0, $this->cursorPosition - 1)
            . mb_substr($this->buffer, $this->cursorPosition);
        $this->cursorPosition--;
        $this->updated = true;
    }

    public function insertAtCursor(string $characters)
    {
        $this->buffer
            = mb_substr($this->buffer, 0, $this->cursorPosition)
            . $characters
            . mb_substr($this->buffer, $this->cursorPosition);
        $this->updated = true;
    }

    public function insertAndAdvance(string $characters)
    {
        $this->insertAtCursor($characters);
        $this->cursorPosition += mb_strlen($characters);
    }

    /**
     * Add a line to the history
     */
    public function addHistory(string $line)
    {
        $hist = $this->history;
        array_unshift($hist, $line);
        $this->history = array_values($hist);
    }

    public function getCursorPosition(): int
    {
        return $this->cursorPosition;
    }

    public function setCursorRelative(int $offs)
    {
        $this->setCursorPosition($this->cursorPosition + $offs);
        $this->updated = true;
    }

    public function setCursorPosition(int $pos)
    {
        if ($pos < 0) {
            $pos = 0;
        } elseif ($pos > mb_strlen($this->buffer)) {
            $pos = mb_strlen($this->buffer);
        }

        $this->cursorPosition = $pos;
        $this->updated = true;
    }

    public function updated(): bool
    {
        $u = $this->updated;
        $this->updated = false;
        return $u;
    }

}