<?php

namespace NoccyLabs\InteractiveShell\Option;

abstract class AbstractOptionSet
{
    /** @var array <string,OptionInterface> */
    private array $options = [];

    protected function bindOption(OptionInterface $option)
    {
        $key = $option->getKey();
        $this->options[$key] = $option;
    }

    public function getKeys(): array
    {
        return array_keys($this->options);
    }
    
    public function getAll(): array
    {
        $k = array_keys($this->options);
        $v = array_map(fn($o) => $o->getValue(), $this->options);
        return array_combine($k, $v);
    }

    public function getOptions(): array
    {
        return $this->options;
    }

    public function get(string $key): mixed
    {
        if (!array_key_exists($key, $this->options)) {
            throw new \Exception("Invalid key");
        }
        $opt = $this->options[$key];

        return $opt->getValue();
    }

    public function set(string $key, mixed $value): void
    {
        if (!array_key_exists($key, $this->options)) {
            throw new \Exception("Invalid key");
        }
        $opt = $this->options[$key];

        $opt->setValueFromLiteral($value);
    }
}
