<?php

namespace NoccyLabs\InteractiveShell\Option;

use NoccyLabs\InteractiveShell\Option\Type\AbstractType;
use NoccyLabs\InteractiveShell\Option\Type\AnyType;
use NoccyLabs\InteractiveShell\Option\Type\BooleanType;
use NoccyLabs\InteractiveShell\Option\Type\FloatType;
use NoccyLabs\InteractiveShell\Option\Type\IntegerType;
use NoccyLabs\InteractiveShell\Option\Type\StringType;

class Option implements OptionInterface
{
    protected string $key;

    protected mixed $value;

    protected mixed $default;

    protected AbstractType $type;

    protected ?string $comment;

    public function __construct(string $key, mixed $value, mixed $default, AbstractType|string $type, ?string $comment=null)
    {
        $this->key = $key;
        $this->value = $value;
        $this->default = $default;
        $this->comment = $comment;

        if (is_string($type)) {
            $this->type = match ($type) {
                'string' => new StringType(),
                'integer' => new IntegerType(),
                'float' => new FloatType(),
                'bool' => new BooleanType(),
                'any' => new AnyType(),
                default => new AnyType(),
            };
        } else {
            $this->type = $type;
        }
    }

    public function getKey(): string
    {
        return $this->key;
    }

    public function resetValue(): void
    {
        $this->setValue($this->default);
    }

    public function getValue(): mixed
    {
        return $this->value;
    }

    public function setValue(mixed $rawValue): void
    {
        $this->value = $rawValue;
    }

    public function setValueFromLiteral(string $newValue): void
    {
        $cast = $this->type->castFromStringValue($newValue);
        $this->setValue($cast);
    }

    public function getComment(): ?string
    {
        return $this->comment;
    }

    public function __toString()
    {
        return $this->type->castToPrintable($this->value);
    }

}