<?php

namespace NoccyLabs\InteractiveShell\Option;

class OptionSet
{
    private array $options = [];

    public function __construct(array $options)
    {
        foreach ($options as $key=>$o) {
            $this->options[$key] = (object)[
                'key' => $key,
                'type' => $o['type']??'any',
                'value' => $o['value']??null,
                'default' => $o['default']??null,
                'comment' => $o['comment']??null,
                'observers' => [],
            ];
        }
    }

    public function observe(string $key, callable $observer)
    {
        if (!array_key_exists($key, $this->options)) {
            throw new \Exception("Invalid key to observe");
        }
        $opt = $this->options[$key];

        if (in_array($observer, $opt->observers, true)) {
            return;
        }

        $opt->observers[] = $observer;
    }

    public function getAll(): array
    {
        $k = array_keys($this->options);
        $v = array_map(fn($o) => $o->value, $this->options);
        return array_combine($k, $v);
    }

    public function get(string $key): mixed
    {
        if (!array_key_exists($key, $this->options)) {
            throw new \Exception("Invalid key");
        }
        $opt = $this->options[$key];

        return $opt->value;
    }

    public function set(string $key, mixed $value): void
    {
        if (!array_key_exists($key, $this->options)) {
            throw new \Exception("Invalid key");
        }
        $opt = $this->options[$key];

        $opt->value = $value;
        foreach ($opt->observers as $observer) {
            call_user_func($observer, $value);
        }
    }
}