<?php

namespace NoccyLabs\InteractiveShell\Option;

use NoccyLabs\InteractiveShell\Option\Attribute\AsOption;
use NoccyLabs\InteractiveShell\Option\Type\AbstractType;
use NoccyLabs\InteractiveShell\Option\Type\AnyType;
use NoccyLabs\InteractiveShell\Option\Type\BooleanType;
use NoccyLabs\InteractiveShell\Option\Type\FloatType;
use NoccyLabs\InteractiveShell\Option\Type\IntegerType;
use NoccyLabs\InteractiveShell\Option\Type\StringType;
use ReflectionClass;
use ReflectionNamedType;
use ReflectionProperty;

class ReflectedOption extends Option
{
    protected object $class;

    protected ReflectionProperty $property;

    protected mixed $notifyCb = null;

    public function __construct(object $class, ReflectionProperty $property, AsOption $attribute)
    {
        $key = $attribute->key;
        $comment = $attribute->info;

        $default = $property->getDefaultValue();
        $value = $property->getValue($class);

        $reflectionType = $property->getType();
        if ($reflectionType instanceof ReflectionNamedType) {
            $type = (string)$reflectionType;
        } else {
            $type = "string";
        }

        parent::__construct($key, $value, $default, $type, $comment);
        
        $this->class = $class;
        $this->property = $property;
        
        // TODO make this part of Option
        if ($notifyCb = $attribute->notify) {
            if (is_string($notifyCb) && !is_callable($notifyCb)) {
                $notifyCb = [ $class, $notifyCb ];
            }
            if (is_callable($notifyCb)) {
                $this->notifyCb = $notifyCb;
            }
        }
    }

    public function getType(): ?string
    {
        return $this->type->getType();
    }

    public function getValue(): mixed
    {
        return $this->property->getValue($this->class);
    }

    public function setValue(mixed $rawValue): void
    {
        $this->property->setValue($this->class, $rawValue);

        // TODO integrate this with the observer logic
        if (is_callable($this->notifyCb)) call_user_func($this->notifyCb);
    }

}