<?php

namespace NoccyLabs\InteractiveShell\Output;

use LogicException;

class ConsoleFormatter
{
    public bool $color = true;

    /**
     * 
     * 
     * @param string $string The string to format
     */
    public function formatString(string $string): string
    {
        return preg_replace_callback("|<([a-z0-9;=/#]+?)>|", function ($m) {
            $v = $m[1];
            $a = [];
            if (str_starts_with($v,"/")) {
                $isClosingTag = true;
                $v = substr($v,1);
                if ($v == "") {
                    return "\e[0m";
                }
            } else {
                $isClosingTag = false;
            }
            $tags = explode(";", $v);
            $eraseLine = false;
            foreach ($tags as $tag) {
                if (str_contains($tag,"=")) {
                    [$key,$val] = explode("=",$tag,2);
                } else {
                    $key = "options";
                    $val = $tag;
                }
                switch ($key) {
                    case 'options':
                        $opts = explode(",",$val);
                        foreach($opts as $opt) {
                            switch ($opt) {
                                case 'bold': $a[] = $isClosingTag?22:1; break;
                                case 'dim': $a[] = $isClosingTag?22:2; break;
                                case 'italic': $a[] = $isClosingTag?23:3; break;
                                case 'underline': $a[] = $isClosingTag?24:4; break;
                                case 'reverse': $a[] = $isClosingTag?27:7; break;
                                case 'el': $eraseLine = true; break;
                            }
                        }
                        break;
                    case 'fg':
                        $color = $this->makeColor($val,true);
                        if (is_array($color)) array_push($a, ...$color);
                        break;
                    case 'bg':
                        $color = $this->makeColor($val,false);
                        if (is_array($color)) array_push($a, ...$color);
                        break;
                }
            }
            return "\e[".join(";",$a)."m" . ($eraseLine?"\e[K":"");
        }, $string);
    }

    private function makeColor(string $color, bool $foreground): ?array
    {
        if (!str_starts_with($color, "#")) {
            return match ($color) {
                'black'     => $foreground?[30]:[40],
                'red'       => $foreground?[31]:[41],
                'green'     => $foreground?[32]:[42],
                'yellow'    => $foreground?[33]:[43],
                'blue'      => $foreground?[34]:[44],
                'magenta'   => $foreground?[35]:[45],
                'cyan'      => $foreground?[36]:[46],
                'white'     => $foreground?[37]:[47],
                'ltblack'  => $foreground?[90]:[100],
                'ltred'    => $foreground?[91]:[101],
                'ltgreen'  => $foreground?[92]:[102],
                'ltyellow' => $foreground?[93]:[103],
                'ltblue'   => $foreground?[94]:[104],
                'ltmagenta'=> $foreground?[95]:[105],
                'ltcyan'   => $foreground?[96]:[106],
                'ltwhite'  => $foreground?[97]:[107],
                default     => throw new LogicException("Invalid color: {$color}"),
            };
        } else {
            if (strlen($color) == 4) {
                if ($color == '#'.$color[1].$color[1].$color[1]) {
                    $g = floor(hexdec(substr($color, 1,1)) / 16 * 24);
                    $c = 232 + $g;
                } else {
                    $r = floor(hexdec(substr($color, 1, 1)) / 16 * 6);
                    $g = floor(hexdec(substr($color, 2, 1)) / 16 * 6);
                    $b = floor(hexdec(substr($color, 3, 1)) / 16 * 6);
                    $c = 16 + (36 * $r) + (6 * $g) + $b;
                }
                return [ $foreground?38:48, 5, $c ];
            }
        }
    }

    /**
     * Return the length of text in a formatted string, sans markup.
     * 
     * @param string $string
     * @return int The number of printable characters
     */
    public function measureLength(string $string): int
    {
        $stripped = preg_replace("<\e\\[[0-9;]+?m>", "", $string);
        return mb_strlen($stripped);
    }


}