<?php

namespace NoccyLabs\InteractiveShell\Output;

use NoccyLabs\InteractiveShell\Input\InputBuffer;

class ConsoleOutput implements OutputInterface
{

    private ConsoleFormatter $consoleFormatter;

    private InputBuffer $inputBuffer;

    private bool $promptVisible = false;

    private int $width;

    public function __construct(ConsoleFormatter $consoleFormatter, InputBuffer $inputBuffer)
    {
        $this->consoleFormatter = $consoleFormatter;
        $this->inputBuffer = $inputBuffer;
        pcntl_signal(SIGWINCH, [ $this, "onWindowResized" ]);
        $this->onWindowResized();
    }

    public function getWidth(): int
    {
        return $this->width;
    }

    public function onWindowResized()
    {
        $this->width = intval(exec("tput cols"));
    }

    public function writeFormatted(string $content)
    {
        $content = $this->consoleFormatter->formatString($content);

        $this->write($content);
    }

    public function write(string $content)
    {
        if ($this->promptVisible) {
            $this->erasePrompt();
        }
        echo rtrim($content,"\n ")."\n";
    }

    public function erasePrompt()
    {
        echo "\r\e[0m\e[K";
        if (!$this->promptVisible) return;
    }

    public function redrawPrompt()
    {
        $prompt = $this->consoleFormatter->formatString($this->inputBuffer->getPrompt());
        $promptLen = $this->consoleFormatter->measureLength($prompt);

        $input = $this->inputBuffer->getBuffer();
        $cp = $promptLen + $this->inputBuffer->getCursorPosition();
        echo "\r\e[K". $prompt . $input . "\e[K\r\e[{$cp}C";
        $this->promptVisible = true;
    }

    public function flushPrompt()
    {
        $prompt = $this->consoleFormatter->formatString($this->inputBuffer->getPrompt());
        $input = $this->inputBuffer->getBuffer();
        echo "\r\e[K". $prompt . $input . "\n";
        $this->promptVisible = false;

    }




}