<?php

namespace NoccyLabs\InteractiveShell\Output;


class ConsoleFormatterTest extends \PHPUnit\Framework\TestCase
{

    /**
     * @dataProvider unformattedAndFormattedStrings
     * @covers NoccyLabs\InteractiveShell\Output\DataProvider
     */
    public function testFormattingStrings(string $string, string $expectedOutput)
    {
        $formatter = new ConsoleFormatter();

        $output = $formatter->formatString($string);
        
        $this->assertEquals($expectedOutput, $output);
    }

    /**
     * @dataProvider formattedStringsWithLengths
     * @covers NoccyLabs\InteractiveShell\Output\DataProvider
     */
    public function testMeasuringFormattedStrings(string $string, int $expectedLength)
    {
        $formatter = new ConsoleFormatter();

        $length = $formatter->measureLength($string);
        
        $this->assertEquals($expectedLength, $length);
    }

    public function formattedStringsWithLengths()
    {
        return [
            [ "hello", 5 ],
            [ "\e[1mhello\e[0m", 5 ],
            [ "\e[33;1mgreen\e[0;31mred\e[0m", 8],
        ];
    }

    public function unformattedAndFormattedStrings()
    {
        return [
            [ "<bold>hello</bold>", "\e[1mhello\e[22m" ],
            [ "<bold>hello</>", "\e[1mhello\e[0m" ],
            [ "<options=bold>hello</>", "\e[1mhello\e[0m" ],
            [ "<fg=red>red</><fg=green>green</>", "\e[31mred\e[0m\e[32mgreen\e[0m"],
            [ "<fg=red>red<fg=green>green</>", "\e[31mred\e[32mgreen\e[0m"],
        ];
    }
}